@@ -5,10 +5,13 @@ import urllib
 import base64
 import hashlib
 import os.path
+import six
+import uuid
+import copy
+from collections import OrderedDict
 
-from django.db import models
+from django.db import models, transaction
 from django import forms
-from django.utils.datastructures import SortedDict
 from django.core.mail import EmailMultiAlternatives
 from django.conf import settings
 from django.utils.translation import ugettext_lazy as _, ugettext
@@ -16,18 +19,17 @@ from django.shortcuts import redirect
 from django.dispatch import receiver
 from django.template.loader import render_to_string
 from django.utils.functional import cached_property
-from django.utils.encoding import python_2_unicode_compatible
+from django.utils.encoding import python_2_unicode_compatible, force_bytes, force_text
 from django.template.defaultfilters import truncatechars
 from django.core.files import File
 from django.core.files.storage import default_storage
 
-from django_extensions.db.fields import UUIDField
 import html2text
 
 from widgy.models import Content, Node
 from widgy.signals import pre_delete_widget
 from widgy.models.mixins import StrictDefaultChildrenMixin, DefaultChildrenMixin, TabbedContainer, StrDisplayNameMixin
-from widgy.utils import update_context, build_url, force_bytes, QuerySet
+from widgy.utils import update_context, build_url, QuerySet, unset_pks
 from widgy.contrib.page_builder.models import Bucket, Html
 from widgy.contrib.page_builder.forms import MiniCKEditorField, CKEditorField
 from .forms import PhoneNumberField
@@ -119,8 +121,8 @@ class RepostHandler(BaseMappingHandler):
         abstract = True
 
     def execute(self, request, form):
-        query_string = urllib.urlencode(self.get_mapping(request, form))
-        urllib.urlopen(self.url_to_post, query_string)
+        query_string = six.moves.urllib.parse.urlencode(self.get_mapping(request, form)).encode('ascii')
+        six.moves.urllib.request.urlopen(self.url_to_post, query_string)
 
 
 class MappingValue(FormElement):
@@ -159,7 +161,7 @@ class FieldMappingValue(StrDisplayNameMixin, MappingValue):
     form = FieldMappingValueForm
     name = models.CharField(max_length=255)
 
-    field_ident = models.CharField(max_length=36)
+    field_ident = models.UUIDField(null=True)
 
     class Meta:
         verbose_name = _('mapped field')
@@ -242,9 +244,10 @@ class EmailSuccessHandlerBase(StrDisplayNameMixin, FormSuccessHandler):
         data = []
         # keep the data in the same order as the form
         for name, field in self.parent_form.get_fields().items():
-            data.append(
-                (field.label, form.cleaned_data[name])
+            serialized_value = field.serialize_value(
+                form.cleaned_data[name]
             )
+            data.append((field.label, serialized_value))
         return render_to_string('widgy/form_builder/form_email.html', {
             'data': data,
             'self': self,
@@ -263,6 +266,7 @@ class EmailSuccessHandlerBase(StrDisplayNameMixin, FormSuccessHandler):
         if self.include_form_data:
             for value in form.cleaned_data.values():
                 if isinstance(value, File):
+                    value.file.seek(0)  # The file has already been read once to save to disk.
                     msg.attach(value.name, value.read(), getattr(value, 'content_type', None))
 
         msg.send()
@@ -312,7 +316,7 @@ class EmailUserHandler(EmailSuccessHandlerBase):
                 " fill out a form. You can customize the body of the email.")
 
     # an input in our form
-    to_ident = models.CharField(_('to'), max_length=36)
+    to_ident = models.UUIDField(_('to'), null=True)
 
     class Meta:
         verbose_name = _('user success email')
@@ -342,7 +346,7 @@ class EmailUserHandler(EmailSuccessHandlerBase):
 @widgy.register
 @python_2_unicode_compatible
 class SubmitButton(StrDisplayNameMixin, FormElement):
-    text = models.CharField(max_length=255, default=lambda: ugettext('submit'), verbose_name=_('text'))
+    text = models.CharField(max_length=255, default=_('submit'), verbose_name=_('text'))
 
     tooltip = _("The submit button for a form.")
 
@@ -441,9 +445,14 @@ def friendly_uuid(uuid):
     A shortend version of a UUID to use when collisions are acceptable.
     The returned string will have 40 bits of entropy assuming a UUID4.
     """
-    result = base64.b32encode(hashlib.sha1(str(uuid)).digest()[:5]).lower()
+    result = base64.b32encode(hashlib.sha1(force_bytes(uuid)).digest()[:5]).lower()
+    unicode_result = result.decode('ascii')
     # avoid accidental profanity
-    return result.replace('e', '0').replace('o', '1').replace('u', '8').replace('i', '9')
+    profanity_filter = (unicode_result.replace('e', '0')
+                                      .replace('o', '1')
+                                      .replace('u', '8')
+                                      .replace('i', '9'))
+    return profanity_filter
 
 
 @widgy.register
@@ -455,7 +464,7 @@ class Form(TabbedContainer, StrDisplayNameMixin, StrictDefaultChildrenMixin, Con
                             help_text=_("A name to help identify this form. Only admins see this."))
 
     # associates instances of the same logical form across versions
-    ident = UUIDField()
+    ident = models.UUIDField(default=uuid.uuid4, editable=False)
 
     editable = True
 
@@ -477,7 +486,9 @@ class Form(TabbedContainer, StrDisplayNameMixin, StrictDefaultChildrenMixin, Con
 
     class Meta:
         verbose_name = _('form')
-        verbose_name_plural = _('forms')
+
+        # this is to set the admin page's title to 'Form Submissions'
+        verbose_name_plural = _('form submissions')
 
     def __str__(self):
         return self.name
@@ -493,8 +504,10 @@ class Form(TabbedContainer, StrDisplayNameMixin, StrictDefaultChildrenMixin, Con
         """
         Returns a django.forms.Form class based on my child widgets.
         """
-        fields = SortedDict((child.get_formfield_name(), child.get_formfield())
-                            for child in self.depth_first_order() if isinstance(child, BaseFormField))
+        fields = OrderedDict(
+            (child.get_formfield_name(), child.get_formfield())
+            for child in self.depth_first_order() if isinstance(child, BaseFormField)
+        )
 
         mixins = []
         for child in self.depth_first_order():
@@ -580,7 +593,7 @@ class Form(TabbedContainer, StrDisplayNameMixin, StrictDefaultChildrenMixin, Con
         """
         A dictionary of formfield name -> FormField widget
         """
-        ret = SortedDict()
+        ret = OrderedDict()
         for child in self.depth_first_order():
             if isinstance(child, FormField):
                 ret[child.get_formfield_name()] = child
@@ -609,10 +622,18 @@ class Form(TabbedContainer, StrDisplayNameMixin, StrictDefaultChildrenMixin, Con
 
     @models.permalink
     def submission_url(self):
-        return ('admin:%s_%s_change' % (self._meta.app_label, self._meta.module_name),
+        return ('admin:%s_%s_change' % (self._meta.app_label, self._meta.model_name),
                 (self.pk,),
                 {})
 
+    @transaction.atomic
+    def clone_new_page(self):
+        new = copy.copy(self)
+        unset_pks(new)
+        new.ident = uuid.uuid4()
+        new.save()
+        return new
+
 
 class BaseFormField(FormElement):
     formfield_class = None
@@ -655,7 +676,7 @@ class FormField(StrDisplayNameMixin, BaseFormField):
 
     help_text = models.TextField(blank=True, verbose_name=_('help text'))
     # associates instances of the same logical field across versions
-    ident = UUIDField()
+    ident = models.UUIDField(default=uuid.uuid4, editable=False)
 
     form = FormFieldForm
 
@@ -681,7 +702,7 @@ class FormField(StrDisplayNameMixin, BaseFormField):
         to store in the db. The return value with also be used in the
         CSV download of form data.
         """
-        return unicode(value)
+        return force_text(value)
 
     @property
     def widget_attrs(self):
@@ -911,11 +932,17 @@ class FileUpload(FormField):
             self.storage.get_valid_name(os.path.basename(filename))
         )
 
+    def _save(self, value):
+        filename = self.generate_filename(value.name)
+        filename = self.storage.save(filename, value)
+        self.filename = self.storage.url(filename)
+
     def serialize_value(self, value):
         if value:
-            filename = self.generate_filename(value.name)
-            filename = self.storage.save(filename, value)
-            return self.storage.url(filename)
+            if not hasattr(self, 'filename'):
+                self._save(value)
+
+            return self.filename
         else:
             return ''
 
@@ -935,7 +962,7 @@ class FormSubmission(models.Model):
 
     created_at = models.DateTimeField(auto_now_add=True)
     form_node = models.ForeignKey(Node, on_delete=models.PROTECT, related_name='form_submissions')
-    form_ident = models.CharField(max_length=Form._meta.get_field('ident', False).max_length)
+    form_ident = models.UUIDField()
 
     class FormSubmissionQuerySet(QuerySet):
         def get_formfield_labels(self):
@@ -949,7 +976,7 @@ class FormSubmission(models.Model):
                 submission__in=self,
             ).values('field_ident').distinct().order_by('field_node__path').values_list('field_ident', flat=True)
 
-            ret = SortedDict([
+            ret = OrderedDict([
                 ('created_at', ugettext('Created at')),
             ])
             for field_uuid in uuids:
@@ -964,8 +991,8 @@ class FormSubmission(models.Model):
 
         def as_ordered_dictionaries(self, order):
             for submission in self.as_dictionaries():
-                yield SortedDict((ident, submission.get(ident, ''))
-                                 for ident in order)
+                yield OrderedDict((ident, submission.get(ident, ''))
+                                  for ident in order)
 
         def to_csv(self, output):
             """
@@ -977,10 +1004,15 @@ class FormSubmission(models.Model):
 
             writer = csv.DictWriter(output, list(headers))
 
-            def encode(d):
-                return dict(
-                    (k, force_bytes(v)) for k, v in d.items()
-                )
+            # python2 csv expects bytes, but python3's works in unicode
+            if six.PY2:
+                def encode(d):
+                    return dict(
+                        (k, force_bytes(v)) for k, v in d.items()
+                    )
+            else:
+                def encode(d):
+                    return d
 
             writer.writerow(encode(headers))
 
@@ -993,7 +1025,7 @@ class FormSubmission(models.Model):
                 form_ident=form.ident,
             )
 
-            for name, field in form.get_fields().iteritems():
+            for name, field in form.get_fields().items():
                 value = field.serialize_value(data[name])
                 submission.values.create(
                     field_node=field.node,
@@ -1008,6 +1040,7 @@ class FormSubmission(models.Model):
     class Meta:
         verbose_name = _('form submission')
         verbose_name_plural = _('form submissions')
+        ordering = ['-created_at']
 
     def as_dict(self):
         ret = {'created_at': self.created_at}
@@ -1029,8 +1062,7 @@ class FormValue(models.Model):
     # field_name is our last resort, in case the field has been deleted.
     field_node = models.ForeignKey(Node, on_delete=models.SET_NULL, null=True)
     field_name = models.CharField(max_length=255)
-    field_ident = models.CharField(
-        max_length=FormField._meta.get_field('ident', False).max_length)
+    field_ident = models.UUIDField()
 
     value = models.TextField()
 
