@@ -5,6 +5,9 @@ from django.utils.translation import ugettext_lazy as _, ugettext
 from django.utils.encoding import python_2_unicode_compatible
 from django.dispatch import receiver
 from django.template.defaultfilters import truncatechars
+from django.contrib.sites.models import Site
+
+from filer.fields.image import FilerImageField
 
 from widgy.models import Content
 from widgy.models.mixins import (
@@ -20,7 +23,7 @@ from widgy.utils import build_url, SelectRelatedManager
 import widgy
 
 
-class Layout(StrictDefaultChildrenMixin, Content):
+class BaseLayout(StrictDefaultChildrenMixin, Content):
     """
     Base class for all layouts.
     """
@@ -35,6 +38,14 @@ class Layout(StrictDefaultChildrenMixin, Content):
         return False
 
 
+class Layout(BaseLayout):
+    """
+    Subclasses of Layout will be used to populate WidgyPage root_choices.
+    """
+    class Meta:
+        abstract = True
+
+
 class Bucket(Content):
     draggable = False
     deletable = False
@@ -51,7 +62,7 @@ class MainContent(Bucket):
 
     @classmethod
     def valid_child_of(cls, parent, obj=None):
-        return isinstance(parent, Layout)
+        return isinstance(parent, BaseLayout)
 
     class Meta:
         verbose_name = _('main content')
@@ -67,7 +78,7 @@ class Sidebar(Bucket):
 
     @classmethod
     def valid_child_of(cls, parent, obj=None):
-        return isinstance(parent, Layout)
+        return isinstance(parent, BaseLayout)
 
     class Meta:
         verbose_name = _('sidebar')
@@ -103,7 +114,7 @@ class Markdown(Content):
 
 
 class HtmlForm(forms.ModelForm):
-    content = CKEditorField(required=False, label=_('Content'))
+    content = CKEditorField(label=_('Content'))
 
 
 @widgy.register
@@ -152,6 +163,7 @@ class CalloutBucket(Bucket):
 @python_2_unicode_compatible
 class Callout(models.Model):
     name = models.CharField(max_length=255, verbose_name=_('name'))
+    site = models.ForeignKey(Site, null=True, blank=True)
     root_node = WidgyField(
         site=settings.WIDGY_MEZZANINE_SITE,
         null=True,
@@ -264,7 +276,8 @@ class Section(StrDisplayNameMixin, Content):
 class Image(Content):
     editable = True
 
-    image = ImageField(verbose_name=_('image'))
+    image = FilerImageField(verbose_name=_('image'), null=True,
+                            related_name='+', on_delete=models.PROTECT)
 
     objects = SelectRelatedManager(select_related=['image'])
 
@@ -485,7 +498,7 @@ class Figure(StrDisplayNameMixin, Content):
     tooltip = _("A figure is a self-contained piece of content. It can be used"
                 " to add a caption to an image or a video for example.")
 
-    position = models.CharField(default=lambda: ugettext('center'), verbose_name=_('position'), max_length=50, choices=[
+    position = models.CharField(default='center', verbose_name=_('position'), max_length=50, choices=[
         ('left', _('Float left')),
         ('right', _('Float right')),
         ('center', _('Center')),
@@ -508,7 +521,7 @@ class Video(StrDisplayNameMixin, Content):
     video = VideoField(verbose_name=_('video'))
 
     editable = True
-    tooltip = _("Add a video to your page. Support YouTube, Vimeo, and"
+    tooltip = _("Add a video to your page. Supports YouTube, Vimeo, and"
                 " others.")
 
     class Meta:
