@@ -5,13 +5,16 @@ from django.conf import settings
 from django.conf.urls import url
 from django import forms
 from django.core.urlresolvers import reverse
-from django.contrib.admin.util import quote
+from django.contrib.admin.utils import quote
 from django.utils.translation import ugettext_lazy as _, ugettext, ungettext
+from django.utils.html import format_html
 from django.utils import timezone
 from django.contrib import messages
 from django.db.models.signals import post_save
 from django.db.models import Min, Q
+from django.contrib.sites.models import Site
 
+from mezzanine.utils.sites import current_site_id
 from mezzanine.pages.admin import PageAdmin
 try:
     from mezzanine.pages.admin import PageAdminForm
@@ -24,8 +27,13 @@ from mezzanine.core.models import (CONTENT_STATUS_PUBLISHED,
 from widgy.forms import WidgyFormMixin, VersionedWidgyWidget
 from widgy.contrib.widgy_mezzanine import get_widgypage_model
 from widgy.contrib.widgy_mezzanine.views import ClonePageView, UnpublishView
-from widgy.utils import format_html
+from widgy.contrib.page_builder.admin import CalloutAdmin
+from widgy.contrib.page_builder.models import Callout
+from widgy.contrib.form_builder.admin import FormAdmin
+from widgy.contrib.form_builder.models import Form
 from widgy.db.fields import get_site
+from widgy.models import Node
+from widgy.admin import WidgyAdmin
 
 
 WidgyPage = get_widgypage_model()
@@ -48,6 +56,7 @@ class WidgyPageAdminForm(WidgyFormMixin, PageAdminForm):
         widgets = {
             'root_node': PageVersionedWidgyWidget,
         }
+        fields = '__all__'
 
     def __init__(self, *args, **kwargs):
         super(WidgyPageAdminForm, self).__init__(*args, **kwargs)
@@ -89,17 +98,31 @@ class WidgyPageAdmin(PageAdmin):
     }
 
     def get_urls(self):
-        clone_view = ClonePageView.as_view(has_permission=self.has_add_permission)
-        unpublish_view = UnpublishView.as_view(has_change_permission=self.has_change_permission)
+        clone_view = ClonePageView.as_view(
+            model=self.model,
+            has_permission=self.has_add_permission,
+        )
+        unpublish_view = UnpublishView.as_view(
+            model=self.model,
+            has_change_permission=self.has_change_permission,
+        )
         return [
-            url('^(.+)/clone/$', self.admin_site.admin_view(clone_view)),
-            url('^(.+)/unpublish/$', self.admin_site.admin_view(unpublish_view)),
+            url(
+                '^(.+)/clone/$',
+                self.admin_site.admin_view(clone_view),
+                name='widgy_mezzanine_widgypage_clone',
+            ),
+            url(
+                '^(.+)/unpublish/$',
+                self.admin_site.admin_view(unpublish_view),
+                name='widgy_mezzanine_widgypage_unpublish',
+            ),
         ] + super(WidgyPageAdmin, self).get_urls()
 
     def _save_and_commit(self, request, obj):
         site = self.get_site()
         commit_model = site.get_version_tracker_model().commit_model
-        if not site.has_add_permission(request, commit_model):
+        if not site.has_add_permission(request, obj, commit_model):
             messages.error(request, _("You don't have permission to commit."))
         else:
             if obj.root_node.has_changes():
@@ -116,7 +139,7 @@ class WidgyPageAdmin(PageAdmin):
     def _save_and_approve(self, request, obj):
         site = self.get_site()
         commit_model = site.get_version_tracker_model().commit_model
-        if not site.has_add_permission(request, commit_model) or \
+        if not site.has_add_permission(request, obj, commit_model) or \
                 not site.has_change_permission(request, commit_model):
             messages.error(request, _("You don't have permission to approve commits."))
         else:
@@ -134,6 +157,7 @@ class WidgyPageAdmin(PageAdmin):
             self._save_and_commit(request, obj)
         elif '_save_and_approve' in request.POST and self.has_review_queue:
             self._save_and_approve(request, obj)
+        request.POST = request.POST.copy()
         request.POST['_continue'] = True
         super(WidgyPageAdmin, self).save_model(request, obj, form, change)
 
@@ -174,7 +198,7 @@ class WidgyPageAdmin(PageAdmin):
         else:
             context['save_buttons'] = self.unreviewed_buttons[status]
         if not add:
-            context['history_url'] =  site.reverse(site.history_view, kwargs={'pk': obj.root_node_id})
+            context['history_url'] = site.reverse(site.history_view, kwargs={'pk': obj.root_node_id})
         return super(WidgyPageAdmin, self).render_change_form(request, context, add, change, form_url, obj, *args, **kwargs)
 
     @property
@@ -232,7 +256,7 @@ class UndeletePageAdminMixin(object):
         if resp.status_code == 302 and resp['Location'].startswith('../'):
             viewname = 'admin:%s_%s_change' % (
                 obj._meta.app_label,
-                obj._meta.module_name)
+                obj._meta.model_name)
             resp['Location'] = reverse(viewname, args=(quote(obj.pk),))
         return resp
 
@@ -256,11 +280,6 @@ class UndeletePage(WidgyPage):
         return super(UndeletePage, self).__init__(*args, **kwargs)
 
 
-# Remove built in Mezzanine models from the admin center
-from mezzanine.pages.models import RichTextPage
-
-admin.site.unregister(RichTextPage)
-
 admin.site.register(WidgyPage, WidgyPageAdmin)
 admin.site.register(UndeletePage, UndeletePageAdmin)
 
@@ -300,14 +319,101 @@ def publish_page_on_approve(sender, instance, created, **kwargs):
             )
 
 
+class MultiSiteFormAdmin(FormAdmin):
+    def get_queryset(self, request):
+        version_tracker_model = self.get_site().get_version_tracker_model()
+        site_pages = version_tracker_model.objects.filter(widgypage__site_id=current_site_id())
+        site_nodes = Node.objects.filter(versiontracker__in=site_pages)
+
+        # This query seems like it could get slow. If that's the case,
+        # something along these lines might be helpful:
+        # Node.objects.all().extra(
+        #     tables=[
+        #         '"widgy_node" AS "root"',
+        #         'widgy_versiontracker',
+        #         'widgy_mezzanine_widgypage',
+        #         'pages_page',
+        #     ],
+        #     where=[
+        #         'root.path = SUBSTR(widgy_node.path, 1, 4)',
+        #         'widgy_versiontracker.id = widgy_mezzanine_widgypage.root_node_id',
+        #         'pages_page.id = widgy_mezzanine_widgypage.page_ptr_id',
+        #         'pages_page.site_id = 1',
+        #     ]
+        # )
+        qs = super(MultiSiteFormAdmin, self).get_queryset(request).filter(
+            _nodes__path__path_root__in=site_nodes.values_list('path'),
+        )
+        return qs
+
+    def get_site(self):
+        return get_site(settings.WIDGY_MEZZANINE_SITE)
+
+admin.site.unregister(Form)
+admin.site.register(Form, MultiSiteFormAdmin)
+
+
+class MultiSiteCalloutAdmin(WidgyAdmin):
+    def get_site_list(self, request):
+        if not hasattr(request, '_site_list'):
+            # Mezzanine has a weird data model for site permissions. This optimizes the query into
+            # one single SQL statement. This also avoids raising an ObjectDoesNotExist error in case
+            # a user does not have a sitepermission object.
+            request._site_list = Site.objects.filter(sitepermission__user=request.user)
+        return request._site_list
+
+    def get_fields(self, request, obj=None):
+        site_list = self.get_site_list(request)
+        if request.user.is_superuser or len(site_list) > 1:
+            return ('name', 'site', 'root_node')
+        else:
+            return ('name', 'root_node')
+
+    def formfield_for_foreignkey(self, db_field, request, **kwargs):
+        if db_field.name == 'site' and not request.user.is_superuser:
+            # See MultiSiteCalloutAdmin.get_fields() about this query
+            kwargs['queryset'] = self.get_site_list(request)
+
+            # Non superusers have to select a site, otherwise the callout will be global.
+            kwargs['required'] = True
+        return super(MultiSiteCalloutAdmin, self).formfield_for_foreignkey(
+            db_field, request, **kwargs)
+
+    def save_model(self, request, obj, form, change):
+        if not change:
+            site_list = self.get_site_list(request)
+            if len(site_list) == 1:
+                obj.site = site_list.get()
+        return super(MultiSiteCalloutAdmin, self).save_model(request, obj, form, change)
+
+    def get_queryset(self, request):
+        qs = super(MultiSiteCalloutAdmin, self).get_queryset(request)
+        if not (request.user.is_superuser or (request.user.is_staff and Site.objects.count() == 1)):
+            qs = qs.filter(site__sitepermission__user=request.user)
+        return qs
+
+admin.site.unregister(Callout)
+admin.site.register(Callout, MultiSiteCalloutAdmin)
+
 if REVIEW_QUEUE_INSTALLED:
     from widgy.contrib.review_queue.admin import VersionCommitAdminBase
-    from widgy.contrib.review_queue.models import ReviewedVersionCommit
+    from widgy.contrib.review_queue.models import ReviewedVersionCommit, ReviewedVersionTracker
 
     class VersionCommitAdmin(VersionCommitAdminBase):
         def get_site(self):
             return get_site(settings.WIDGY_MEZZANINE_SITE)
 
+        def get_queryset(self, request):
+            qs = super(VersionCommitAdmin, self).get_queryset(request)
+
+            if not request.user.is_superuser:
+                sites = Site.objects.filter(sitepermission__user=request.user)
+                qs = qs.filter(
+                    tracker__in=ReviewedVersionTracker.objects.filter(widgypage__site__in=sites)
+                )
+
+            return qs
+
     admin.site.register(ReviewedVersionCommit, VersionCommitAdmin)
 
     site = get_site(settings.WIDGY_MEZZANINE_SITE)
