@@ -1,10 +1,11 @@
 from haystack import indexes
+from django.utils.encoding import force_text
 
 from widgy.contrib.widgy_mezzanine import get_widgypage_model
 from widgy.templatetags.widgy_tags import render_root
 from widgy.utils import html_to_plaintext
 
-from .signals import widgypage_pre_index
+from widgy.signals import widgy_pre_index
 
 WidgyPage = get_widgypage_model()
 
@@ -18,7 +19,7 @@ class PageIndex(indexes.SearchIndex, indexes.Indexable):
     text = indexes.CharField(document=True)
 
     def full_prepare(self, *args, **kwargs):
-        widgypage_pre_index.send(sender=self)
+        widgy_pre_index.send(sender=self)
         return super(PageIndex, self).full_prepare(*args, **kwargs)
 
     def get_model(self):
@@ -36,4 +37,4 @@ class PageIndex(indexes.SearchIndex, indexes.Indexable):
                          content])
 
     def prepare_keywords(self, obj):
-        return [unicode(k) for k in obj.keywords.all()]
+        return [force_text(k) for k in obj.keywords.all()]
