@@ -1,28 +1,22 @@
+import six
+
 from django.db import models
-from django.db.models.fields.related import ReverseSingleRelatedObjectDescriptor
-from django.db.models.loading import get_app
+from django.apps import apps
 from django.contrib.contenttypes.models import ContentType
 from django.utils.functional import SimpleLazyObject
+from django.db.models.fields.related import ForwardManyToOneDescriptor
 
-# WidgyContentType has a patched get_for_models that doesn't ignore proxy
-# models
-from widgy.generic.models import ContentType as WidgyContentType
 from widgy.utils import fancy_import, update_context
 
-from south.modelsinspector import add_introspection_rules
-
-add_introspection_rules([], ["^widgy\.db.fields\.WidgyField"])
-add_introspection_rules([], ["^widgy\.db.fields\.VersionedWidgyField"])
-
 
 def get_site(site):
-    if isinstance(site, basestring):
+    if isinstance(site, six.string_types):
         return fancy_import(site)
     else:
         return site
 
 
-class WidgyFieldObjectDescriptor(ReverseSingleRelatedObjectDescriptor):
+class WidgyFieldObjectDescriptor(ForwardManyToOneDescriptor):
     """
     .. note::
         we need to transport the ContentType of the Node all the way to
@@ -30,19 +24,26 @@ class WidgyFieldObjectDescriptor(ReverseSingleRelatedObjectDescriptor):
         know if the rest of the model is valid yet.
     """
     def __set__(self, instance, value):
-        """
-        """
         if isinstance(value, ContentType):
-            ModelClass = self.field.rel.to
-            value, value._ct = ModelClass(), value
-
-        super(WidgyFieldObjectDescriptor, self).__set__(instance, value)
+            setattr(instance, self.field.pre_save_ct_field_name, value)
+        else:
+            super(WidgyFieldObjectDescriptor, self).__set__(instance, value)
+            if hasattr(instance, self.field.pre_save_ct_field_name):
+                delattr(instance, self.field.pre_save_ct_field_name)
 
 
 class WidgyField(models.ForeignKey):
     """
     Model field that inherits from ``models.ForeignKey``.  Contains validation
     and context switching that is needed for Widgy fields.
+
+    A WidgyField can be assigned a ContentType instance in order to create a
+    new root node of that content type. ::
+
+        my_content_type = ContentType.objects.get_for_model(Layout)
+        my_model = MyModel(
+            root_node=my_content_type
+        )
     """
     def __init__(self, site=None, to=None, root_choices=None, **kwargs):
         if to is None:
@@ -64,22 +65,24 @@ class WidgyField(models.ForeignKey):
         super(WidgyField, self).contribute_to_class(cls, name)
         setattr(cls, self.name, WidgyFieldObjectDescriptor(self))
 
+    @property
+    def pre_save_ct_field_name(self):
+        return '_widgy_field_pre_save_ct_{0}'.format(self.name)
+
     def add_root(self, model_instance, root_content_kwargs={}):
-        value = getattr(model_instance, self.name)
-        return value._ct.model_class().add_root(self.site, **root_content_kwargs).node
+        # This is a weird API. It would be nicer to take the content class of
+        # the new root node, but add_root is a public interface
+        # (django-widgy-blog is using it).
+        content_type = getattr(model_instance, self.pre_save_ct_field_name)
+        return content_type.model_class().add_root(self.site, **root_content_kwargs).node
 
     def pre_save(self, model_instance, add):
         """
-        Relies on WidgyFieldObjectDescriptor to set the content type on an
-        unsaved throwaway Node so that we know how to properly instantiate a
-        real node later on.
+        When adding a new root, we might be passed a ContentType that we need
+        to turn into a root node. Relies on WidgyFieldObjectDescriptor to set
+        the content type in pre_save_ct_field_name.
         """
-        try:
-            value = getattr(model_instance, self.name)
-        except models.ObjectDoesNotExist:
-            value = None
-
-        if hasattr(value, '_ct'):
+        if hasattr(model_instance, self.pre_save_ct_field_name):
             node = self.add_root(model_instance)
             setattr(model_instance, self.name, node)
             setattr(model_instance, self.attname, node.pk)
@@ -110,11 +113,11 @@ class WidgyField(models.ForeignKey):
 
             # we cannot use models.get_model because this class could be
             # abstract.
-            return getattr(get_app(app_label), model_name)
+            return getattr(apps.get_app_config(app_label).models_module, model_name)
 
         layouts = tuple(map(normalize, layouts))
         classes = (cls for cls in self.site.get_all_content_classes()
-                   if issubclass(cls, layouts))
+                   if self.site.valid_root_of(self, cls, layouts))
 
         # This method must return a queryset, because it is used as
         # choices for a ModelChoiceField.
@@ -124,7 +127,7 @@ class WidgyField(models.ForeignKey):
         # test databases haven't been set up yet, but we try to query
         # the ContentType table.
         return ContentType.objects.filter(pk__in=[
-            ct.pk for ct in WidgyContentType.objects.get_for_models(
+            ct.pk for ct in ContentType.objects.get_for_models(
                 for_concrete_models=False,
                 *classes
             ).values()])
@@ -154,7 +157,7 @@ class WidgyField(models.ForeignKey):
         # the root node, it isn't saved so it can't have a pk. The base
         # field's validate will fail, even though pre_save will actually
         # create a node and set the fk field.
-        if value is None and not self.blank and getattr(model_instance, self.name):
+        if value is None and not self.blank and hasattr(model_instance, self.pre_save_ct_field_name):
             return
         else:
             return super(WidgyField, self).validate(value, model_instance)
