@@ -8,13 +8,15 @@ import logging
 import itertools
 import copy
 
-from django.db import models
+from django.db import models, transaction
 from django import forms
 from django.forms.models import modelform_factory, ModelForm
 from django.contrib.contenttypes.models import ContentType
 from django.template import RequestContext
+from django.template.loader import render_to_string
 from django.contrib.admin import widgets
 from django.template.defaultfilters import capfirst
+from django.utils.encoding import force_text, python_2_unicode_compatible
 
 from treebeard.mp_tree import MP_Node
 
@@ -26,9 +28,7 @@ from widgy.exceptions import (
 )
 from widgy.signals import pre_delete_widget
 from widgy.generic import WidgyGenericForeignKey, ProxyGenericRelation
-from widgy.utils import (
-    exception_to_bool, update_context, render_to_string, force_text, unset_pks,
-)
+from widgy.utils import exception_to_bool, update_context, unset_pks
 from widgy.widgets import DateTimeWidget, DateWidget, TimeWidget
 
 logger = logging.getLogger(__name__)
@@ -44,6 +44,7 @@ FORMFIELD_FOR_DBFIELD_DEFAULTS = {
 }
 
 
+@python_2_unicode_compatible
 class Node(MP_Node):
     """
     Instances of this class maintain the Materialized Path tree structure that
@@ -69,8 +70,8 @@ class Node(MP_Node):
         app_label = 'widgy'
         unique_together = [('content_type', 'content_id')]
 
-    def __unicode__(self):
-        return unicode(self.content)
+    def __str__(self):
+        return force_text(self.content)
 
     def to_json(self, site):
         children = [c.to_json(site) for c in self.get_children()]
@@ -176,20 +177,16 @@ class Node(MP_Node):
             contents[node.content_type_id].add(node.content_id)
 
         # Convert that mapping to content_types -> Content instances
-        for content_type_id, content_ids in contents.iteritems():
-            try:
-                ct = ContentType.objects.get_for_id(content_type_id)
-                model_class = ct.model_class()
-            except AttributeError:
-                # get_for_id raises AttributeError when there's no model_class in django < 1.6.
-                model_class = None
+        for content_type_id, content_ids in contents.items():
+            ct = ContentType.objects.get_for_id(content_type_id)
+            model_class = ct.model_class()
             if model_class:
                 contents[content_type_id] = ct.model_class().objects.in_bulk(content_ids)
             else:
                 ct = ContentType.objects.get(id=content_type_id)
                 contents[content_type_id] = dict((id, UnknownWidget(ct, id)) for id in content_ids)
                 # Warn about using an UnknownWidget. It doesn't matter which instance we use.
-                next(contents[content_type_id].itervalues(), UnknownWidget(ct, None)).warn()
+                next(iter(contents[content_type_id].values()), UnknownWidget(ct, None)).warn()
         return contents
 
     @classmethod
@@ -257,9 +254,9 @@ class Node(MP_Node):
         """
 
         validator = partial(site.validate_relationship, self.content)
-        return filter(
+        return list(filter(
             exception_to_bool(validator, ParentChildRejection),
-            classes)
+            classes))
 
     def filter_child_classes_recursive(self, site, classes):
         """
@@ -287,7 +284,8 @@ class Node(MP_Node):
         my_family = set(self.depth_first_order())
         return [i for i in all_nodes if validator(i.content) and i not in my_family]
 
-    def clone_tree(self, freeze=True):
+    @transaction.atomic(savepoint=False)
+    def clone_tree(self, freeze=True, new_page=False):
         """
         1. new_root <- root_node
         2. new_root.content <- Clone(root_node.content)
@@ -302,18 +300,23 @@ class Node(MP_Node):
         """
         # This method only supports cloning an entire tree. We don't need it
         # for versioning, and I'm not sure what the semantics would be.
+        if new_page:
+            content_clone_method = 'clone_new_page'
+        else:
+            content_clone_method = 'clone'
+
         cls = self.__class__
         assert self.depth == 1
         self.maybe_prefetch_tree()
         new_root = cls.add_root(
-            content=self.content.clone(),
+            content=getattr(self.content, content_clone_method)(),
             numchild=self.numchild,
             is_frozen=freeze,
         )
         children_to_create = []
         for child in self.depth_first_order()[1:]:
             children_to_create.append(Node(
-                content=child.content.clone(),
+                content=getattr(child.content, content_clone_method)(),
                 path=new_root.path + child.path[cls.steplen:],
                 is_frozen=freeze,
                 depth=child.depth,
@@ -326,18 +329,22 @@ class Node(MP_Node):
         if self.is_frozen:
             raise InvalidOperation({'message': "This widget is uneditable."})
 
+    @transaction.atomic(savepoint=False)
     def delete(self, *args, **kwargs):
         self.check_frozen()
         return super(Node, self).delete(*args, **kwargs)
 
+    @transaction.atomic(savepoint=False)
     def add_child(self, *args, **kwargs):
         self.check_frozen()
         return super(Node, self).add_child(*args, **kwargs)
 
+    @transaction.atomic(savepoint=False)
     def add_sibling(self, *args, **kwargs):
         self.check_frozen()
         return super(Node, self).add_sibling(*args, **kwargs)
 
+    @transaction.atomic(savepoint=False)
     def move(self, *args, **kwargs):
         self.check_frozen()
         return super(Node, self).move(*args, **kwargs)
@@ -433,7 +440,7 @@ class Content(models.Model):
 
     def get_api_url(self, site):
         return site.reverse(site.content_view, kwargs={
-            'object_name': self._meta.module_name,
+            'object_name': self._meta.model_name,
             'app_label': self._meta.app_label,
             'object_pk': self.pk})
 
@@ -444,7 +451,7 @@ class Content(models.Model):
             '__class__': self.class_name,
             'css_classes': self.get_css_classes(),
             'component': self.component_name,
-            'model': self._meta.module_name,
+            'model': self._meta.model_name,
             'object_name': self._meta.object_name,
             'draggable': self.draggable,
             'deletable': self.deletable,
@@ -463,9 +470,19 @@ class Content(models.Model):
 
     def get_attributes(self):
         model_data = {}
-        for field in self._meta.fields:
+        for field in self._meta.concrete_fields:
             if field.serialize:
                 model_data[field.attname] = field.value_from_object(self)
+        for field in self._meta.many_to_many:
+            # this is copied from django.forms.models.model_to_dict
+            if self.pk is None:
+                model_data[field.name] = []
+            else:
+                qs = field.value_from_object(self)
+                if qs._result_cache is not None:
+                    model_data[field.name] = [item.pk for item in qs]
+                else:
+                    model_data[field.name] = list(qs.values_list('pk', flat=True))
         return model_data
 
     @classmethod
@@ -474,7 +491,7 @@ class Content(models.Model):
         :Returns: a json-able python object that represents the class type.
         """
         return {
-            '__class__': "%s.%s" % (cls._meta.app_label, cls._meta.module_name),
+            '__class__': "%s.%s" % (cls._meta.app_label, cls._meta.model_name),
             'title': capfirst(cls._meta.verbose_name),
             'css_classes': cls.get_class_css_classes(),
             'tooltip': cls.tooltip and force_text(cls.tooltip),
@@ -482,20 +499,20 @@ class Content(models.Model):
 
     @property
     def display_name(self):
-        return unicode(self._meta.verbose_name)
+        return force_text(self._meta.verbose_name)
 
     @property
     def class_name(self):
         """
-        :Returns: a fully qualified classname including app_label and module_name
+        :Returns: a fully qualified classname including app_label and model_name
         """
-        return "%s.%s" % (self._meta.app_label, self._meta.module_name)
+        return "%s.%s" % (self._meta.app_label, self._meta.model_name)
 
     @classmethod
     def get_class_css_classes(cls):
         if hasattr(cls, 'css_classes'):
             return cls.css_classes
-        return (cls._meta.app_label, cls._meta.module_name)
+        return (cls._meta.app_label, cls._meta.model_name)
 
     def get_css_classes(self):
         if hasattr(self, 'css_classes'):
@@ -546,6 +563,7 @@ class Content(models.Model):
         defaults = {
             "form": self.form,
             "formfield_callback": partial(self.formfield_for_dbfield, request=request),
+            "fields": "__all__",
         }
         form_class = modelform_factory(self.__class__, **defaults)
         # Rather than make everybody subclass a special form, add the
@@ -581,6 +599,7 @@ class Content(models.Model):
         return True
 
     @classmethod
+    @transaction.atomic
     def add_root(cls, site, **kwargs):
         """
         Creates a new Content instance, stores it in the database, and calls
@@ -591,6 +610,7 @@ class Content(models.Model):
         obj.post_create(site)
         return obj
 
+    @transaction.atomic
     def add_child(self, site, cls, **kwargs):
         self.check_frozen()
         obj = cls.objects.create(**kwargs)
@@ -605,6 +625,7 @@ class Content(models.Model):
         obj.post_create(site)
         return obj
 
+    @transaction.atomic
     def add_sibling(self, site, cls, **kwargs):
         self.check_frozen()
         if self.node.is_root():
@@ -634,7 +655,7 @@ class Content(models.Model):
     @classmethod
     def get_templates_hierarchy(cls, **kwargs):
         templates = kwargs.get('hierarchy', (
-            'widgy/{app_label}/{module_name}/{template_name}{extension}',
+            'widgy/{app_label}/{model_name}/{template_name}{extension}',
             'widgy/{app_label}/{template_name}{extension}',
             'widgy/{template_name}{extension}',
         ))
@@ -657,7 +678,7 @@ class Content(models.Model):
     def get_template_kwargs(cls, **kwargs):
         defaults = {
             'app_label': cls._meta.app_label,
-            'module_name': cls._meta.module_name,
+            'model_name': cls._meta.model_name,
         }
         defaults.update(**kwargs)
 
@@ -691,7 +712,7 @@ class Content(models.Model):
         if not context:
             context = RequestContext(request)
         with update_context(context, {'form': self.get_form(request, prefix=self.get_form_prefix())}):
-            return render_to_string(template or self.edit_templates, context)
+            return render_to_string(template or self.edit_templates, context.flatten())
 
     def get_preview_template(self, site):
         """
@@ -713,7 +734,7 @@ class Content(models.Model):
         with update_context(context, {'self': self}):
             return render_to_string(
                 template or self.get_render_templates(context),
-                context
+                context.flatten(),
             )
 
     def formfield_for_dbfield(self, db_field, **kwargs):
@@ -723,7 +744,7 @@ class Content(models.Model):
 
         If kwargs are given, they're passed to the form Field's constructor.
         """
-        request = kwargs.pop("request", None)
+        request = kwargs.pop("request")
         formfield = db_field.formfield(**kwargs)
 
         if formfield and isinstance(db_field, (models.ForeignKey, models.ManyToManyField)):
@@ -746,6 +767,7 @@ class Content(models.Model):
             'edit_template': self.get_form_template(request),
         }
 
+    @transaction.atomic
     def reposition(self, site, right=None, parent=None):
         self.check_frozen()
         old_parent_node = self.node.get_parent()
@@ -784,6 +806,7 @@ class Content(models.Model):
             site.validate_relationship(self, c)
             c._recheck_children(site)
 
+    @transaction.atomic
     def delete(self, raw=False):
         self.check_frozen()
         pre_delete_widget.send(self.__class__, instance=self, raw=raw)
@@ -792,6 +815,14 @@ class Content(models.Model):
         self.node.delete()
         super(Content, self).delete()
 
+    def clone_new_page(self):
+        """
+        Clone the content for the purposes of cloning an entire page. This is
+        useful to reset form submissions, for example.
+        """
+        return self.clone()
+
+    @transaction.atomic
     def clone(self):
         """
         **Note:** In order for clone to work, you need to have an
@@ -804,7 +835,20 @@ class Content(models.Model):
         # See https://code.djangoproject.com/ticket/4027
         new = copy.copy(self)
         unset_pks(new)
+        try:
+            # Contents keep a cached copy of their Node. After the clone the
+            # cache is invalid.
+            del new._node
+        except AttributeError:
+            pass
         new.save()
+
+        for f in self._meta.many_to_many:
+            # This will only handle simple many-to-manies, those that don't
+            # have a custom through table.
+            f.save_form_data(new, f.value_from_object(self))
+
+        assert new != self
         return new
 
     def save(self, *args, **kwargs):
