@@ -3,6 +3,7 @@ import copy
 from django.db import models
 from django.utils import timezone
 from django.utils.functional import cached_property
+from django.utils.encoding import python_2_unicode_compatible
 from django.db.models.deletion import ProtectedError
 from django.conf import settings
 from django.template.defaultfilters import date as date_format
@@ -12,6 +13,7 @@ from widgy.models.base import Node
 from widgy.utils import QuerySet, unset_pks
 
 
+@python_2_unicode_compatible
 class VersionCommit(models.Model):
     tracker = models.ForeignKey('VersionTracker', related_name='commits')
     parent = models.ForeignKey('VersionCommit', null=True, on_delete=models.PROTECT)
@@ -29,7 +31,7 @@ class VersionCommit(models.Model):
     def is_published(self):
         return self.publish_at <= timezone.now()
 
-    def __unicode__(self):
+    def __str__(self):
         if self.message:
             subject = " - '%s'" % self.message.strip().split('\n')[0]
         else:
@@ -56,10 +58,13 @@ class VersionTracker(models.Model):
             """
 
             filters = {}
-            for rel_obj in (self.model._meta.get_all_related_objects() +
-                            self.model._meta.get_all_related_many_to_many_objects()):
-                if not issubclass(rel_obj.model, VersionCommit):
-                    name = rel_obj.field.rel.related_name or rel_obj.var_name
+            rel_objs = [
+                f for f in self.model._meta.get_fields(include_hidden=True)
+                if (f.many_to_many or f.one_to_many or f.one_to_one) and f.auto_created
+            ]
+            for rel_obj in rel_objs:
+                if not issubclass(rel_obj.field.model, VersionCommit):
+                    name = rel_obj.field.related_query_name()
                     filters[name + '__isnull'] = True
             return self.filter(**filters)
 
@@ -185,19 +190,24 @@ class VersionTracker(models.Model):
         """
         Names of reverse relationships of WidgyFields that point to us.
         """
-        for rel_obj in cls._meta.get_all_related_objects():
+        rel_objs = [
+            f for f in cls._meta.get_fields()
+            if (f.one_to_many or f.one_to_one)
+            and f.auto_created and not f.concrete
+        ]
+        for rel_obj in rel_objs:
             if isinstance(rel_obj.field, WidgyField):
                 yield rel_obj.get_accessor_name()
 
     @cached_property
     def owners(self):
-        return list(owner
-                    for attr in self.get_owner_related_names()
-                    for owner in getattr(self, attr).all())
+        return [owner
+                for attr in self.get_owner_related_names()
+                for owner in getattr(self, attr).all()]
 
-    def clone(self):
+    def clone(self, new_page=False):
         vt = copy.copy(self)
-        vt.working_copy = vt.working_copy.clone_tree(freeze=False)
+        vt.working_copy = vt.working_copy.clone_tree(freeze=False, new_page=new_page)
         commits = list(self._commits_to_clone())
         unset_pks(vt)
         vt.head = None
