@@ -18,14 +18,32 @@ define([ 'jquery' ], function(jQuery){
     return cookieValue;
   })('csrftoken');
 
+  if (!csrftoken) {
+    // The CSRF token was not set, perhaps they are using
+    // CSRF_COOKIE_HTTPONLY.
+    csrftoken = $('[name=csrfmiddlewaretoken]').val();
+  }
+
+  if (!csrftoken) {
+    console.error(
+      "Widgy couldn't figure out the CSRF token. It checked two places:\n" +
+      " - The csrftoken cookie (not available if CSRF_COOKIE_HTTPONLY is set" +
+      " to True\n" +
+      " - The csrfmiddlewaretoken hidden input provided by the csrftoken" +
+      " template tag\n" +
+      " If you can change neither of these problems, please open an issue on" +
+      " http://github.com/fusionbox/django-widgy/issues explaining your" +
+      " situation."
+    );
+  }
+
   function csrfSafeMethod(method) {
     // these HTTP methods do not require CSRF protection
     return (/^(GET|HEAD|OPTIONS|TRACE)$/.test(method));
   }
   $.ajaxSetup({
-    crossDomain: false, // obviates need for sameOrigin test
     beforeSend: function(xhr, settings) {
-      if (!csrfSafeMethod(settings.type)) {
+      if (!csrfSafeMethod(settings.type) && !this.crossDomain) {
         xhr.setRequestHeader("X-CSRFToken", csrftoken);
       }
     }
